#define F_CPU 9600000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>

/*
  Timing optimized for 8Mhz AVR (excl. XMEGA and reduced instruction set)
  The total length of each bit is 1.25µs (10 cycles @ 8Mhz)
  * At 0µs the dataline is pulled high.  (cycle 4)
  * To send a zero the dataline is pulled low after 0.375µs (4+3=7 cycles).
  * To send a one the dataline is pulled low after 0.625µs (4+5=9 cycles).
  10 cycles can not be reached without loop unrolling, or by using byte
  writes to the ports. However since critical the timing between the rising and 
  falling edge is correct, it seems to be acceptable to slightly increase 
  bit timing
  
 Final timing: 
  * 11 cycles for bits 7-1
  * 17 cycles for bit 0    
*/

#define ws2812_port 0x18  // Number of the data port register
#define ws2812_pin 0    // Number of the data out pin
#define WS2811_PORT PORTB
#define WS2811_PIN 0
uint8_t eemode EEMEM=0;
volatile uint8_t mode=1;
volatile uint16_t prell=0;

void ws2812_sendarray_8Mhz(uint8_t *data,uint16_t datlen)
{

  uint8_t curbyte,ctr;
  
  while (datlen--) {
    curbyte=*data++;

    asm volatile("    						ldi  %0,8    \n\t"
								"loop%=:lsl  %1      \n\t"
								"    dec  %0      \n\t"
								"    sbi  %2,  %3    \n\t"
								"    brcs .+2    \n\t"
								"    cbi  %2,  %3    \n\t"
								"    brcc .+2    \n\t"
								"    cbi  %2,  %3    \n\t"
								"    brne loop%=    \n\t"
								:  "=&d" (ctr)
								:  "r" (curbyte), "I" (ws2812_port), "I" (ws2812_pin)
    );
    
    // loop overhead including byte load is 6+1 cycles
  }
}


uint8_t LEDdata[3*4]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
uint8_t r;
uint8_t x=0;

void muster0(){

for(r=0;r<100;r++){
	LEDdata[0]=LEDdata[0+3]=LEDdata[0+3*2]=LEDdata[0+3*3]=r;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(6);
	}
		LEDdata[0]=LEDdata[0+3]=LEDdata[0+3*2]=LEDdata[0+3*3]=0;
	for(r=0;r<100;r++){

	LEDdata[1]=LEDdata[1+3]=LEDdata[1+3*2]=LEDdata[1+3*3]=r;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(6);
	}
	LEDdata[1]=LEDdata[1+3]=LEDdata[1+3*2]=LEDdata[1+3*3]=0;
	for(r=0;r<100;r++){
	LEDdata[2]=LEDdata[2+3]=LEDdata[2+3*2]=LEDdata[2+3*3]=r;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(6);
	}
	LEDdata[2]=LEDdata[2+3]=LEDdata[2+3*2]=LEDdata[2+3*3]=0;
}
void muster1(){

	for(r=0;r<255;r++){
	LEDdata[x]=LEDdata[x+3]=LEDdata[x+3*2]=LEDdata[x+3*3]=r;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(6);
	}
	for(r=255;r>0;r--){
	LEDdata[x]=LEDdata[x+3]=LEDdata[x+3*2]=LEDdata[x+3*3]=r;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(6);
	}
	
}


void muster2(){


	LEDdata[0]=TCNT0>>1;
	LEDdata[0+3]=LEDdata[0+3*2]^TCNT0;
	LEDdata[0+3*2]=LEDdata[0+3*3]^TCNT0;
	LEDdata[0+3*3]=LEDdata[1]^TCNT0;
	LEDdata[1]=LEDdata[1+3]^TCNT0;
	LEDdata[1+3]=LEDdata[1+3*2]^TCNT0;
	LEDdata[1+3*2]=LEDdata[1+3*3]^TCNT0;
	LEDdata[1+3*3]=LEDdata[2]^TCNT0;
	LEDdata[2]=LEDdata[2+3]^TCNT0;
	LEDdata[2+3]=LEDdata[2+3*2]^TCNT0;
	LEDdata[2+3*2]=LEDdata[2+3*3]^TCNT0;
	LEDdata[2+3*3]=LEDdata[0]^TCNT0;
	ws2812_sendarray_8Mhz(LEDdata,3*4);
	_delay_ms(60);

}

ISR(INT0_vect){
if(prell==0){mode++;
eeprom_write_byte(&eemode, mode);
LEDdata[0]=LEDdata[0+3]=LEDdata[0+3*2]=LEDdata[0+3*3]=0;
LEDdata[1]=LEDdata[1+3]=LEDdata[1+3*2]=LEDdata[1+3*3]=0;
LEDdata[2]=LEDdata[2+3]=LEDdata[2+3*2]=LEDdata[2+3*3]=0;
prell=20;}



}
ISR(TIM0_OVF_vect){
if(prell)prell--;



}

void main(){
DDRB=1;
PORTB|=(1<<1);//pullup taster
MCUCR|=(1<<ISC01);//falling edge
GIMSK|=(1<<INT0);
TCCR0B|=(1<<CS02)|(1<<CS00);
TIMSK0|=(1<<TOIE0);
OSCCAL-=40; // ASM is for 8Mhz, callibrate Oscillator to 8MhZ
sei();
mode = eeprom_read_byte (&eemode);
while(1){


switch (mode){
case 0: muster0();break;
case 1: x=0;muster1();break;
case 2: x=1;muster1();break;
case 3: x=2;muster1();break;
case 4: muster2();break;
default: mode=0;break;


}



}
}

